=begin
(c) TIG 2010
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
___________________________________________________________________________

  cofgreporter.rb >>> ../Plugins/ folder
___________________________________________________________________________
Usage:
Use this after adding some CofG's [i.e. 'Center of Gravity' tags]
Type 'cofgreporter' in the Ruby Console.
If the Model SKP is new and has not yet been saved you will get an error.
If there are no CofG's to Report you will get an error.
If the destination CofG Report cannot be written - 
e.g. it's already open in Excel - you will get an error.
Otherwise a CofG Report is written into the Model's folder.
It is named to match the Model - 
e.g. 'MyModel.skp' and 'MyModel_CofG_Report.csv'
A '.csv' file can be opened in Excel etc.
Link the .csv into a pre-formated .xls file with additional 'sums' etc, so 
that it auto-updates.
It is sorted alphanumerically and reports
'_ID' 'Weight' 'Volume' - the final two are in the units used in the CofG's.
The '_ID' is the CofG name-tag based on the original Instance's name, but 
remember that you might have adjusted this in the dialogs during the making 
of the CofG's - i.e. it is as displayed in the CofG's text-tag, not 
necessarily the Instance's name.
___________________________________________________________________________
Donations:
by PayPal to info @ revitrev.org
___________________________________________________________________________
Version:
1.0 20101009 First release.
1.1 20101010 csv defined outside of begin/rescue.
1.2 20101010 Extra rescue error messages added.
1.3 21101010 Glitch with some users' "group.definition" fixed.
___________________________________________________________________________
=end
require 'sketchup.rb'
def cofgreporter()
  model=Sketchup.active_model
  mpath=model.path
  if not mpath or mpath==""
    UI.beep
    puts("This 'Untitled' new Model must be Saved\nbefore making a CofG Report !\nExiting.")
    return nil
  end
  defns=model.definitions
  cogs=[]
  defns.each{|d|d.instances.each{|i|cogs << i if i.get_attribute("CofG","tag",false)}if d.group?}
  if not cogs[0]
    UI.beep
    puts("NO CofG's to Report !\nExiting.")
    return nil
  end#if
  mpath=File.dirname(mpath)
  mname=model.title
  rname=mname+"_CofG_Report.csv"
  csv=File.join(mpath,rname)
  begin
    list=[]
    cogs.each{|c|
      i=nil
      c.entities.each{|e|i=e if e.get_attribute("CofG","id",nil)}
      next if not i
      id=i.get_attribute("CofG","id","").to_s.tr(",",".")
      we=i.get_attribute("CofG","weight","").to_s.tr(",",".")
      vo=i.get_attribute("CofG","volume","").to_s.tr(",",".")
      list << id+","+we+","+vo
    }
    file=File.new(csv,"w")
    file.puts("_ID,Weight,Volume\n")
    list.sort.each{|i|file.puts(i+"\n")}
    file.close
    UI.beep
    puts("CofG Report written to\n'"+csv+"'")
  rescue Exception => e
    UI.beep
    UI.beep
    puts e
    puts("CoG Report File:\n'"+csv+"'\nCannot be written - it's probably already open.\nClose it and try making the Report again...\nExiting.")
  end
end#if
